@@ -25,7 +25,7 @@ class AbstractSICConnector(object):
         """
         topics = ['events', 'browser_button', 'detected_person', 'recognised_face', 'audio_language', 'audio_intent',
                   'audio_newfile', 'picture_newfile', 'detected_emotion',
-                  'robot_audio_loaded'  # , memory_data, gui_data?
+                  'robot_audio_loaded', 'memory_data',
                   'robot_posture_changed', 'robot_awake_changed', 'robot_battery_charge_changed',
                   'robot_charging_changed', 'robot_hot_device_detected', 'robot_motion_recording',
                   'text_transcript', 'text_sentiment', 'corona_check', 'detected_object', 'depth_estimated',
@@ -52,8 +52,16 @@ class AbstractSICConnector(object):
                                          'action_idle', 'action_turn', 'action_wakeup', 'action_rest',
                                          'action_set_breathing', 'action_posture', 'action_stiffness',
                                          'action_play_motion', 'action_record_motion',  # 'action_motion_file',
-                                         'action_led_color', 'action_led_animation'],
-            self.device_types['speaker']: ['audio_language', 'action_say', 'action_say_animated', 'action_play_audio',
+                                         'action_led_color', 'action_led_animation',
+                                         'memory_create_interactant', 'memory_set_session',
+                                         'memory_set_entry', 'memory_get_entry', 'memory_get_entries',
+                                         'memory_get_all_entries',
+                                         'memory_set_interactant_data', 'memory_get_interactant_data',
+                                         'memory_get_interactant_data_all', 'memory_get_all_interactants',
+                                         'memory_set_interactant', 'memory_set_dialog_history',
+                                         'memory_get_dialog_history_all',
+                                         'memory_delete_interactant', 'memory_delete_all_interactants'],
+            self.device_types['speaker']: ['audio_language', 'action_say', 'action_speech_param', 'action_say_animated', 'action_play_audio',
                                            'action_stop_talking', 'action_load_audio', 'action_clear_loaded_audio',
                                            'text_to_speech', 'tts_key', 'tts_voice'],
             self.device_types['browser']: ['render_html']
@@ -161,6 +169,12 @@ class AbstractSICConnector(object):
         See: http://doc.aldebaran.com/2-8/family/nao_technical/contact-sensors_naov6.html"""
         pass
 
+    def on_memory_data(self, key: str, value: str) -> None:
+        """
+        Triggered when the robot_memory service sends a memory_data packet.
+        """
+        pass
+
     def on_posture_changed(self, posture: str) -> None:
         """
         Trigger when the posture has changed.
@@ -392,6 +406,12 @@ class AbstractSICConnector(object):
         """Tell the robot (and any recognition module) to stop looking."""
         self.__send('action_video', '-1;0')
 
+    def set_speech_param(self, param: str, value: float) -> None:
+        """Set various speech parameters. These parameters can be pitchShift, doubleVoice, doubleVoiceLevel, doubleVoiceTimeShift, speed, defaultVoiceSpeed.
+        Check http://doc.aldebaran.com/2-8/naoqi/audio/altexttospeech-api.html#ALTextToSpeechProxy::setParameter__ssCR.floatCR for the values these parameters can have
+        """
+        self.__send('action_speech_param', f'{param};{value}')
+
     def say(self, text: str) -> None:
         """A string that the robot should say (in the currently selected language!).
         A TextStarted event will be sent when the speaking starts and a TextDone event after it is finished."""
@@ -583,6 +603,55 @@ class AbstractSICConnector(object):
         """
         self.__send('action_record_motion', 'stop')
 
+    ###########################
+    # Memory actions          #
+    ###########################
+
+    def create_interactant(self, interactant_id: str):
+        self.__send('memory_create_interactant', interactant_id)
+
+    def set_memory_session(self, interactant_id: str, session_id: str):
+        self.__send('memory_set_session', f'{interactant_id};{session_id}')
+
+    def set_memory_entry(self, interactant_id: str, entry_type: str, entry_data: dict):
+        self.__send('memory_set_entry', f'{interactant_id};{entry_type};{dumps(entry_data)}')
+
+    def get_memory_entry(self, interactant_id: str, entry_type: str, entry_id: str):
+        self.__send('memory_get_entry', f'{interactant_id};{entry_type};{entry_id}')
+
+    def get_memory_entries(self, interactant_id: str, entry_type: str):
+        self.__send('memory_get_entries', f'{interactant_id};{entry_type}')
+
+    def get_all_memory_entries(self, entry_type: str):
+        self.__send('memory_get_all_entries', entry_type)
+
+    def set_interactant_data(self, interactant_id: str, key: str, value: str):
+        self.__send('memory_set_interactant_data', f'{interactant_id};{key};{value}')
+
+    def get_interactant_data(self, interactant_id: str, key: str):
+        self.__send('memory_get_interactant_data', f'{interactant_id};{key}')
+
+    def get_interactant_data_all(self, interactant_id: str):
+        self.__send('memory_get_interactant_data_all', interactant_id)
+
+    def get_all_interactants(self):
+        self.__send('memory_get_all_interactants', '')
+
+    def set_interactant(self, interactant_id: str, data: dict):
+        self.__send('memory_set_interactant', f'{interactant_id};{dumps(data)}')
+
+    def delete_interactant(self, interactant_id: str):
+        self.__send('memory_delete_interactant', interactant_id)
+
+    def delete_all_interactants(self):
+        self.__send('memory_delete_all_interactants', '')
+
+    def set_dialog_history(self, interactant_id: str, minidialog_id: str):
+        self.__send('memory_set_dialog_history', f'{interactant_id};{minidialog_id}')
+
+    def get_dialog_history_all(self):
+        self.__send('memory_get_dialog_history_all', '')
+
     ###########################
     # Browser Actions         #
     ###########################
@@ -626,6 +695,9 @@ class AbstractSICConnector(object):
         elif name == 'text_to_speech':
             for speaker in self.devices[self.device_types['speaker']]:
                 pipe.publish(name, speaker)
+        elif name == 'robot_memory':
+            for memory in self.devices[self.device_types['robot']]:
+                pipe.publish(name, memory)
         else:
             print('Unknown service: ' + name)
         pipe.execute()
@@ -659,6 +731,9 @@ class AbstractSICConnector(object):
 
         if channel == 'events':
             self.on_event(event=data.decode('utf-8'))
+        elif channel == 'memory_data':
+            key, value = data.decode('utf-8').split(';')
+            self.on_memory_data(key, value)
         elif channel == 'browser_button':
             self.on_browser_button(button=data.decode('utf-8'))
         elif channel == 'detected_person':
