@@ -110,6 +110,7 @@ class BasicSICConnector(AbstractSICConnector):
         self.__action_listeners = {}
         self.__vision_listeners = {}
         self.__event_listeners = {}
+        self.__memory_listeners = {}
         self.__conditions = []
         self.__loaded_actions = []
         self.robot_state = {'posture': RobotPosture.UNKNOWN,
@@ -152,6 +153,9 @@ class BasicSICConnector(AbstractSICConnector):
         self.__notify_action_listeners(event)
         self.__notify_event_listeners(event)
 
+    def on_memory_data(self, key: str, value: str) -> None:
+        self.__notify_memory_listeners(key, value)
+
     def on_posture_changed(self, posture: str) -> None:
         self.__notify_action_listeners('onPostureChanged', posture)
         self.robot_state['posture'] = RobotPosture[posture.upper()]
@@ -338,11 +342,11 @@ class BasicSICConnector(AbstractSICConnector):
 
         return success_callback, fail_callback, lock
 
-    @staticmethod
-    def __build_shortcut_callback(lock: Event, shortcuts: dict, embedded_callback: callable):
+    def __build_shortcut_callback(self, lock: Event, shortcuts: dict, embedded_callback: callable):
         def shortcut_callback(transcript: str):
             for answer in shortcuts.keys():
                 if any(variant in transcript for variant in shortcuts[answer]):
+                    self.stop_listening()
                     if not lock.is_set():
                         lock.set()
                         if embedded_callback:
@@ -511,7 +515,8 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).say_text_to_speech, text,
                               callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
-    def set_language(self, language_key: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def set_language(self, language_key: str, callback: callable = None,
+                     sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_language, language_key,
                               callback=callback, event='LanguageChanged', sync=sync, load=load)
 
@@ -523,7 +528,8 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).set_non_idle,
                               callback=callback, event='SetNonIdle', sync=sync, load=load)
 
-    def start_looking(self, seconds: int, channels: int = 1, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def start_looking(self, seconds: int, channels: int = 1, callback: callable = None,
+                      sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_looking, seconds, channels,
                               callback=callback, event='WatchingStarted', sync=sync, load=load)
 
@@ -555,7 +561,8 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).play_audio, audio_file,
                               callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
-    def play_loaded_audio(self, audio_identifier: int, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def play_loaded_audio(self, audio_identifier: int, callback: callable = None,
+                          sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).play_loaded_audio, audio_identifier,
                               callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
@@ -575,13 +582,15 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).set_head_color, color,
                               callback=callback, event='HeadColourDone', sync=sync, load=load)
 
-    def set_led_color(self, leds: list, colors: list, duration: int = 0, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def set_led_color(self, leds: list, colors: list, duration: int = 0, callback: callable = None,
+                      sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_led_color, leds, colors, duration,
                               callback=callback, event='LedColorDone', sync=sync, load=load)
     
-    def start_led_animation(self, led_group: str, anim_type: str, colors: list, speed: int,
-                            real_blink=False, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
-        self.__process_action(super(BasicSICConnector, self).start_led_animation, led_group, anim_type, colors, speed, real_blink,
+    def start_led_animation(self, led_group: str, anim_type: str, colors: list, speed: int, real_blink=False,
+                            callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+        self.__process_action(super(BasicSICConnector, self).start_led_animation,
+                              led_group, anim_type, colors, speed, real_blink,
                               callback=callback, event='LedAnimationDone', sync=sync, load=load)
 
     def stop_led_animation(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
@@ -602,9 +611,11 @@ class BasicSICConnector(AbstractSICConnector):
 
     def set_breathing(self, enable: bool, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_breathing, enable,
-                              callback=callback, event='Breathing' + ('Enabled' if enable else 'Disabled'), sync=sync, load=load)
+                              callback=callback, event='Breathing' + ('Enabled' if enable else 'Disabled'),
+                              sync=sync, load=load)
 
-    def go_to_posture(self, posture: Enum, speed: int = 100, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def go_to_posture(self, posture: Enum, speed: int = 100, callback: callable = None,
+                      sync: bool = True, load: bool = False) -> None:
         if callback:
             callback = partial(self.__posture_callback, target_posture=posture, embedded_callback=callback)
         self.__process_action(super(BasicSICConnector, self).go_to_posture, posture.value, speed,
@@ -616,7 +627,8 @@ class BasicSICConnector(AbstractSICConnector):
         else:  # if the posture was not reached
             embedded_callback(False)  # call the listener to signal a failure
 
-    def set_stiffness(self, joints: list, stiffness: int, duration: int = 1000, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def set_stiffness(self, joints: list, stiffness: int, duration: int = 1000, callback: callable = None,
+                      sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_stiffness, joints, stiffness, duration,
                               callback=callback, event='SetStiffnessDone', sync=sync, load=load)
 
@@ -630,7 +642,8 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).play_motion, motion,
                               callback=callback, event='PlayMotionDone', sync=sync, load=load)
 
-    def start_record_motion(self, joint_chains: list, framerate: int = 5, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
+    def start_record_motion(self, joint_chains: list, framerate: int = 5, callback: callable = None,
+                            sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_record_motion, joint_chains, framerate,
                               callback=callback, event='RecordMotionStarted', sync=sync, load=load)
 
@@ -642,6 +655,79 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).browser_show, html,
                               callback=None, event='', sync=sync, load=load)
 
+    ###########################
+    # Memory action           #
+    ###########################
+
+    def create_interactant(self, interactant_id: str, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).create_interactant, interactant_id,
+                              callback=callback, event='InteractantCreated', sync=sync, load=load)
+
+    def set_memory_session(self, interactant_id: str, session_id: str, callback: callable = None,
+                           sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).set_memory_session, interactant_id, session_id,
+                              callback=callback, event='SessionSet', sync=sync, load=load)
+
+    def set_memory_entry(self, interactant_id: str, entry_type: str, entry_data: dict,
+                         callback: callable = None, sync: bool = True, load: bool = False) -> None:
+        self.__process_action(super(BasicSICConnector, self).set_memory_session, interactant_id, entry_type, entry_data,
+                              callback=callback, event='MemoryEntryStored', sync=sync, load=load, memory=True)
+
+    def get_memory_entry(self, interactant_id: str, entry_type: str, entry_id: str,
+                         callback: callable = None, sync: bool = True, load: bool = False) -> None:
+        self.__process_action(super(BasicSICConnector, self).get_memory_entry, interactant_id, entry_type, entry_id,
+                              callback=callback, event=f'Entry_{entry_type}_{entry_id}', sync=sync, load=load, memory=True)
+
+    def get_memory_entries(self, interactant_id: str, entry_type: str, callback: callable = None,
+                           sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_memory_entries, interactant_id, entry_type,
+                              callback=callback, event=f'Entries_{entry_type}', sync=sync, load=load, memory=True)
+
+    def get_all_memory_entries(self, entry_type: str, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_all_memory_entries, entry_type,
+                              callback=callback, event=f'AllEntries_{entry_type}', sync=sync, load=load, memory=True)
+
+    def set_interactant_data(self, interactant_id: str, key: str, value: str, callback: callable = None,
+                             sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).set_interactant_data, interactant_id, key, value,
+                              callback=callback, event='InteractantDataSet', sync=sync, load=load)
+
+    def get_interactant_data(self, interactant_id: str, key: str, callback: callable = None,
+                             sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_interactant_data, interactant_id, key,
+                              callback=callback, event=key, sync=sync, load=load, memory=True)
+
+    def get_interactant_data_all(self, interactant_id: str, callback: callable = None,
+                                 sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_interactant_data_all, interactant_id,
+                              callback=callback, event=f'InteractantData_{interactant_id}', sync=sync, load=load, memory=True)
+
+    def get_all_interactants(self, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_all_interactants,
+                              callback=callback, event='AllInteractants', sync=sync, load=load, memory=True)
+
+    def set_interactant(self, interactant_id: str, data: dict, callback: callable = None,
+                        sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).set_interactant, interactant_id, data,
+                              callback=callback, event='InteractantSet', sync=sync, load=load)
+
+    def set_dialog_history(self, interactant_id: str, minidialog_id: str, callback: callable = None,
+                           sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).set_dialog_history, interactant_id, minidialog_id,
+                              callback=callback, event='DialogHistorySet', sync=sync, load=load)
+
+    def get_dialog_history_all(self, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).get_dialog_history_all,
+                              callback=callback, event='AllDialogHistory', sync=sync, load=load, memory=True)
+
+    def delete_interactant(self, interactant_id: str, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).delete_interactant, interactant_id,
+                              callback=callback, event='InteractantDeleted', sync=sync, load=load)
+
+    def delete_all_interactants(self, callback: callable = None, sync: bool = True, load: bool = False):
+        self.__process_action(super(BasicSICConnector, self).delete_all_interactants,
+                              callback=callback, event='AllInteractantsDeleted', sync=sync, load=load)
+
     ###########################
     # Listeners Management  #
     ###########################
@@ -666,6 +752,13 @@ class BasicSICConnector(AbstractSICConnector):
         if event in self.__event_listeners:
             del self.__event_listeners[event]
 
+    def __register_memory_listener(self, data_key: str, callback: callable) -> None:
+        self.__memory_listeners[data_key] = callback
+
+    def __unregister_memory_listener(self, data_key: str) -> None:
+        if data_key in self.__memory_listeners:
+            del self.__memory_listeners[data_key]
+
     def __register_condition(self, condition: Condition) -> None:
         """
         Subscribe a threading.Condition object that will be notified each time a registered callback is called.
@@ -703,6 +796,12 @@ class BasicSICConnector(AbstractSICConnector):
             listener(*args)
             self.__notify_conditions()
 
+    def __notify_memory_listeners(self, data_key: str, data_value: str) -> None:
+        if data_key in self.__memory_listeners:
+            listener = self.__memory_listeners[data_key]
+            listener(data_value)
+            self.__notify_conditions()
+
     def __notify_conditions(self) -> None:
         for condition in self.__conditions:
             with condition:
@@ -719,7 +818,8 @@ class BasicSICConnector(AbstractSICConnector):
 
         return callback, lock
 
-    def __process_action(self, action: callable, *args, callback: callable, event: str, sync: bool, load: bool):
+    def __process_action(self, action: callable, *args, callback: callable, event: str, sync: bool, load: bool,
+                         memory: bool = False):
         inner_callback = callback
         lock = None
 
@@ -727,7 +827,10 @@ class BasicSICConnector(AbstractSICConnector):
             inner_callback, lock = self.__build_sync_callback(callback)
 
         if inner_callback:
-            self.__register_action_listener(event, inner_callback)
+            if memory:
+                self.__register_memory_listener(event, inner_callback)
+            else:
+                self.__register_action_listener(event, inner_callback)
 
         if load:
             self.__loaded_actions.append(Action(action, *args, lock=lock))
@@ -778,3 +881,4 @@ class BasicSICConnector(AbstractSICConnector):
         self.__conditions = []
         self.__vision_listeners = {}
         self.__event_listeners = {}
+        self.__memory_listeners = {}
